@@ -8,10 +8,13 @@ from collective.exportimport.interfaces import IPathBlobsMarker
 from collective.exportimport.interfaces import IRawRichTextMarker
 from operator import itemgetter
 from plone import api
+from plone.app.layout.viewlets.content import ContentHistoryViewlet
 from plone.i18n.normalizer.interfaces import IIDNormalizer
 from plone.restapi.interfaces import ISerializeToJson
-from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
+from plone.restapi.serializer.converters import json_compatible
 from Products.CMFPlone.interfaces.constrains import ENABLED
+from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
+from Products.CMFPlone.utils import safe_unicode
 from Products.Five import BrowserView
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from zope.component import getMultiAdapter
@@ -105,6 +108,7 @@ class ExportContent(BrowserView):
         include_blobs=1,
         download_to_server=False,
         migration=True,
+        include_revisions=False,
     ):
         self.portal_type = portal_type or []
         if isinstance(self.portal_type, str):
@@ -133,6 +137,7 @@ class ExportContent(BrowserView):
             ("1", "as base-64 encoded strings"),
             ("2", "as blob paths"),
         )
+        self.include_revisions = include_revisions
 
         self.update()
 
@@ -179,6 +184,7 @@ class ExportContent(BrowserView):
                 directory = cfg.clienthome
             filepath = os.path.join(directory, filename)
             with open(filepath, "w") as f:
+                self.start()
                 for number, datum in enumerate(content_generator, start=1):
                     if number == 1:
                         f.write("[")
@@ -198,9 +204,11 @@ class ExportContent(BrowserView):
                 noLongerProvides(self.request, IBase64BlobsMarker)
             elif self.include_blobs == 2:
                 noLongerProvides(self.request, IPathBlobsMarker)
+            self.finish()
             self.request.response.redirect(self.request["ACTUAL_URL"])
         else:
             with tempfile.TemporaryFile(mode="w+") as f:
+                self.start()
                 for number, datum in enumerate(content_generator, start=1):
                     if number == 1:
                         f.write("[")
@@ -225,11 +233,18 @@ class ExportContent(BrowserView):
                 elif self.include_blobs == 2:
                     noLongerProvides(self.request, IPathBlobsMarker)
                 f.seek(0)
+                self.finish()
                 return response.write(safe_bytes(f.read()))
 
     def update(self):
         """Hook to do something before export."""
 
+    def start(self):
+        """Hook to do something before export starts."""
+
+    def finish(self):
+        """Hook to do something after export finishes."""
+
     def build_query(self):
         query = {
             "portal_type": self.portal_type,
@@ -269,7 +284,11 @@ class ExportContent(BrowserView):
 
             if not index % 100:
                 logger.info(u"Handled {} items...".format(index))
-            obj = brain.getObject()
+            try:
+                obj = brain.getObject()
+            except Exception as e:
+                logger.exception(u"Error getting brain %s", brain.getPath(), exc_info=True)
+                continue
             obj = self.global_obj_hook(obj)
             if not obj:
                 continue
@@ -282,6 +301,9 @@ class ExportContent(BrowserView):
                     item = serializer()
                 item = self.fix_url(item, obj)
                 item = self.export_constraints(item, obj)
+                item = self.export_workflow_history(item, obj)
+                item = self.export_revisions(item, obj)
+
                 if self.migration:
                     item = self.update_data_for_migration(item, obj)
                 item = self.global_dict_hook(item, obj)
@@ -296,7 +318,7 @@ class ExportContent(BrowserView):
 
                 yield item
             except Exception as e:
-                logger.exception(u"Error exporting {}".format(obj.absolute_url()))
+                logger.exception(u"Error exporting %s", obj.absolute_url(), exc_info=True)
 
     def portal_types(self):
         """A list with info on all content types with existing items."""
@@ -313,7 +335,7 @@ class ExportContent(BrowserView):
                         "number": number,
                         "value": fti.id,
                         "title": translate(
-                            fti.title, domain="plone", context=self.request
+                            safe_unicode(fti.title), domain="plone", context=self.request
                         ),
                     }
                 )
@@ -439,6 +461,43 @@ class ExportContent(BrowserView):
             }
         return item
 
+    def export_workflow_history(self, item, obj):
+        results = {}
+        workflow_history = getattr(obj.aq_base, "workflow_history", {})
+        for workflow, history in workflow_history.items():
+            results[workflow] = json_compatible(history)
+        if results:
+            item["workflow_history"] = results
+        return item
+
+    def export_revisions(self, item, obj):
+        if not self.include_revisions:
+            return item
+        repo_tool = api.portal.get_tool("portal_repository")
+        history_metadata = repo_tool.getHistoryMetadata(obj)
+        serializer = getMultiAdapter((obj, self.request), ISerializeToJson)
+        content_history_viewlet = ContentHistoryViewlet(obj, self.request, None, None)
+        content_history_viewlet.navigation_root_url = ""
+        content_history_viewlet.site_url = ""
+        full_history = content_history_viewlet.fullHistory() or []
+        history = [i for i in full_history if i["type"] == "versioning"]
+        if not history or len(history) == 1:
+            return item
+        item["exportimport.versions"] = {}
+        # don't export the current version again
+        for history_item in history[1:]:
+            version_id = history_item["version_id"]
+            item_version = serializer(include_items=False, version=version_id)
+            item_version = self.update_data_for_migration(item_version, obj)
+            item["exportimport.versions"][version_id] = item_version
+            # inject metadata (missing for Archetypes content):
+            comment = history_metadata.retrieve(version_id)["metadata"]["sys_metadata"]["comment"]
+            if comment and comment != item["exportimport.versions"][version_id].get("changeNote"):
+                item["exportimport.versions"][version_id]["changeNote"] = comment
+        # current changenote
+        item["changeNote"] = history_metadata.retrieve(-1)["metadata"]["sys_metadata"]["comment"]
+        return item
+
 
 def fix_portal_type(portal_type):
     normalizer = getUtility(IIDNormalizer)
