@@ -9,6 +9,7 @@ from plone import api
 from plone.app.discussion.comment import Comment
 from plone.app.discussion.interfaces import IConversation
 from plone.app.portlets.interfaces import IPortletTypeInterface
+from plone.app.redirector.interfaces import IRedirectionStorage
 from plone.portlets.interfaces import IPortletAssignmentMapping
 from plone.portlets.interfaces import IPortletAssignmentSettings
 from plone.portlets.interfaces import ILocalPortletAssignmentManager
@@ -18,6 +19,7 @@ from Products.Five import BrowserView
 from Products.ZCatalog.ProgressHandler import ZLogHandler
 from zope.annotation.interfaces import IAnnotations
 from zope.component import getUtility
+from zope.component import queryUtility
 from zope.component import queryMultiAdapter
 from zope.component.interfaces import IFactory
 from zope.container.interfaces import INameChooser
@@ -469,7 +471,6 @@ class ImportDefaultPages(BrowserView):
 
     def __call__(self, jsonfile=None, return_json=False):
         if jsonfile:
-            self.portal = api.portal.get()
             status = "success"
             try:
                 if isinstance(jsonfile, str):
@@ -502,17 +503,32 @@ class ImportDefaultPages(BrowserView):
             obj = api.content.get(UID=item["uuid"])
             if not obj:
                 if item["uuid"] == config.SITE_ROOT:
-                    obj = self.portal
+                    obj = api.portal.get()
                 else:
                     continue
-            old = obj.getDefaultPage()
-            if six.PY2:
-                obj.setDefaultPage(item["default_page"].encode("utf-8"))
+            if "default_page_uuid" in item:
+                default_page_obj = api.content.get(UID=item["default_page_uuid"])
+                if not default_page_obj:
+                    logger.info("Default page missing: %s", item["default_page_uuid"])
+                    continue
+                default_page = default_page_obj.id
             else:
-                obj.setDefaultPage(item["default_page"])
-            if old != obj.getDefaultPage():
-                logger.debug(u"Set {} as default page for {}".format(item["default_page"], obj.absolute_url()))
-                results += 1
+                # fallback for old export versions
+                default_page = item["default_page"]
+            if default_page not in obj:
+                logger.info(u"Default page not a child: %s not in %s", default_page, obj.absolute_url())
+                continue
+
+            if default_page == "index_html":
+                # index_html is automatically used as default page
+                continue
+
+            if six.PY2:
+                obj.setDefaultPage(default_page.encode("utf-8"))
+            else:
+                obj.setDefaultPage(default_page)
+            logger.debug(u"Set %s as default page for %s", default_page, obj.absolute_url())
+            results += 1
         return results
 
 
@@ -665,7 +681,10 @@ def register_portlets(obj, item):
     request = getRequest()
     results = 0
     for manager_name, portlets in item.get("portlets", {}).items():
-        manager = getUtility(IPortletManager, manager_name)
+        manager = queryUtility(IPortletManager, manager_name)
+        if not manager:
+            logger.info(u"No portlet manager {}".format(manager_name))
+            continue
         mapping = queryMultiAdapter((obj, manager), IPortletAssignmentMapping)
         namechooser = INameChooser(mapping)
 
@@ -673,12 +692,15 @@ def register_portlets(obj, item):
             # 1. Create the assignment
             assignment_data = portlet_data["assignment"]
             portlet_type = portlet_data["type"]
-            portlet_factory = getUtility(IFactory, name=portlet_type)
+            portlet_factory = queryUtility(IFactory, name=portlet_type)
+            if not portlet_factory:
+                logger.info(u"No factory for portlet {}".format(portlet_type))
+                continue
+
             assignment = portlet_factory()
 
             name = namechooser.chooseName(None, assignment)
             mapping[name] = assignment
-            logger.info("Added portlet {} to {}".format(name, obj.absolute_url()))
 
             # aq-wrap it so that complex fields will work
             assignment = assignment.__of__(site)
@@ -699,16 +721,27 @@ def register_portlets(obj, item):
                 # deserialize data (e.g. for RichText)
                 deserializer = queryMultiAdapter((field, obj, request), IFieldDeserializer)
                 if deserializer is not None:
-                    value = deserializer(value)
+                    try:
+                        value = deserializer(value)
+                    except Exception as e:
+                        logger.info(u"Could not import portlet data {} for field {} on {}: {}".format(
+                            value, field, obj.absolute_url(), str(e)
+                        ))
+                        continue
                 field.set(assignment, value)
 
+            logger.info(u"Added {} '{}' to {} of {}".format(
+                portlet_type, name, manager_name, obj.absolute_url()))
             results += 1
 
     for blacklist_status in item.get("blacklist_status", []):
         status = blacklist_status["status"]
         manager_name = blacklist_status["manager"]
         category = blacklist_status["category"]
-        manager = getUtility(IPortletManager, manager_name)
+        manager = queryUtility(IPortletManager, manager_name)
+        if not manager:
+            logger.info("No portlet manager {}".format(manager_name))
+            continue
         assignable = queryMultiAdapter((obj, manager), ILocalPortletAssignmentManager)
         if status.lower() == "block":
             assignable.setBlacklistStatus(category, True)
@@ -716,3 +749,42 @@ def register_portlets(obj, item):
             assignable.setBlacklistStatus(category, False)
 
     return results
+
+
+def import_plone_redirects(jsondata):
+    storage = getUtility(IRedirectionStorage)
+
+    for key, value in jsondata.items():
+        storage.add(key, value)
+
+
+class ImportRedirects(BrowserView):
+    """Import redirects"""
+
+    def __call__(self, jsonfile=None, return_json=False):
+        if jsonfile:
+            status = "success"
+            try:
+                if isinstance(jsonfile, str):
+                    return_json = True
+                    data = json.loads(jsonfile)
+                elif isinstance(jsonfile, FileUpload):
+                    data = json.loads(jsonfile.read())
+                else:
+                    raise ("Data is neither text nor upload.")
+            except Exception as e:
+                status = "error"
+                logger.error(e)
+                api.portal.show_message(
+                    u"Failure while uploading: {}".format(e),
+                    request=self.request,
+                )
+            else:
+                import_plone_redirects(data)
+                msg = u"Redirects imported"
+                api.portal.show_message(msg, self.request)
+            if return_json:
+                msg = {"state": status, "msg": msg}
+                return json.dumps(msg)
+
+        return self.index()
