@@ -9,8 +9,11 @@ from plone.app.testing import login
 from plone.app.testing import SITE_OWNER_NAME
 from plone.app.testing import SITE_OWNER_PASSWORD
 from plone.app.testing import TEST_USER_ID
+from zope.annotation.interfaces import IAnnotations
+from zope.lifecycleevent import modified
 
 import json
+import six
 import transaction
 import unittest
 
@@ -78,6 +81,7 @@ class TestExport(unittest.TestCase):
         self.assertIn("Export members", browser.contents)
         self.assertIn("Export local roles", browser.contents)
         self.assertIn("Export default pages", browser.contents)
+        self.assertIn("Export redirects", browser.contents)
         self.assertIn("Export object positions", browser.contents)
         # We cannot choose a portal_type, because there is no content to export.
         fti = self.layer['portal'].portal_types['Plone Site']
@@ -310,7 +314,7 @@ class TestExport(unittest.TestCase):
         folder1 = api.content.create(
             container=portal, type="Folder", id="folder1", title="Folder 1"
         )
-        api.content.create(
+        doc1 = api.content.create(
             container=folder1, type="Document", id="doc1", title="Document 1"
         )
         folder1._setProperty("default_page", "doc1")
@@ -324,7 +328,7 @@ class TestExport(unittest.TestCase):
         data = json.loads(contents)
         self.assertListEqual(
             data,
-            [{"default_page": "doc1", "uuid": folder1.UID()}],
+            [{"default_page": "doc1", "uuid": folder1.UID(), 'default_page_uuid': doc1.UID()}],
         )
 
     def test_export_defaultpage_for_site(self):
@@ -332,7 +336,7 @@ class TestExport(unittest.TestCase):
         app = self.layer["app"]
         portal = self.layer["portal"]
         login(app, SITE_OWNER_NAME)
-        api.content.create(
+        doc1 = api.content.create(
             container=portal, type="Document", id="doc1", title="Document 1"
         )
         portal._setProperty("default_page", "doc1")
@@ -346,7 +350,7 @@ class TestExport(unittest.TestCase):
         data = json.loads(contents)
         self.assertListEqual(
             data,
-            [{"default_page": "doc1", "uuid": config.SITE_ROOT}],
+            [{"default_page": "doc1", "uuid": config.SITE_ROOT, "default_page_uuid": doc1.UID()}],
         )
 
     def test_export_ordering(self):
@@ -409,3 +413,132 @@ class TestExport(unittest.TestCase):
         self.assertEqual(data_uuid2order[doc3.UID()], 0)
         self.assertEqual(data_uuid2order[doc1.UID()], 1)
         self.assertEqual(data_uuid2order[doc2.UID()], 2)
+
+    def test_export_redirects_empty(self):
+        browser = self.open_page("@@export_redirects")
+        browser.getForm(action="@@export_redirects").submit(name="form.submitted")
+        contents = browser.contents
+        if not browser.contents:
+            contents = DATA[-1]
+        data = json.loads(contents)
+        self.assertDictEqual(data, {})
+
+    def test_export_redirects(self):
+        # First create some content.
+        app = self.layer["app"]
+        portal = self.layer["portal"]
+        login(app, SITE_OWNER_NAME)
+        api.content.create(
+            container=portal, type="Document", id="doc1", title="Document 1"
+        )
+        api.content.rename(obj=portal['doc1'], new_id='doc1-moved')
+        api.content.create(
+            container=portal, type="Document", id="doc2", title="Document 2"
+        )
+        api.content.rename(obj=portal['doc2'], new_id='doc2-moved')
+        transaction.commit()
+
+        browser = self.open_page("@@export_redirects")
+        browser.getForm(action="@@export_redirects").submit(name="form.submitted")
+        contents = browser.contents
+        if not browser.contents:
+            contents = DATA[-1]
+        data = json.loads(contents)
+        self.assertDictEqual(
+            data,
+            {u'/plone/doc1': u'/plone/doc1-moved',
+            u'/plone/doc2': u'/plone/doc2-moved'},
+        )
+
+    def test_export_versions(self):
+        app = self.layer["app"]
+        portal = self.layer["portal"]
+        request = self.layer["request"]
+        login(app, SITE_OWNER_NAME)
+        if six.PY2:
+            # in Plone 4.3 this is somehow not set...
+            IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"initial_version_changeNote"
+        doc1 = api.content.create(
+            container=portal,
+            type="Document",
+            id="doc1",
+            title="Document 1",
+            description="A Description",
+        )
+        folder1 = api.content.create(
+            container=portal,
+            type="Folder",
+            id="folder1",
+            title="Folder 1",
+        )
+        doc2 = api.content.create(
+            container=folder1,
+            type="Document",
+            id="doc2",
+            title="Document 2",
+            description="A Description",
+        )
+
+        doc1.title= u"Document 1 with changed title"
+        modified(doc1)
+        doc2.title= u"Document 2 with changed title"
+        IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"Föö bar"
+        modified(doc2)
+
+        doc2.description= u"New description in revision 3"
+        IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"I am new!"
+        modified(doc2)
+        folder1.title = u"Folder 1 with changed title"
+        modified(folder1)
+
+        transaction.commit()
+
+        # Now export complete portal.
+        browser = self.open_page("@@export_content")
+        portal_type = browser.getControl(name="portal_type")
+        self.assertEqual(portal_type.value, [])
+        portal_type.value = ["Folder", "Document"]
+
+        depth = browser.getControl(name="depth")
+        self.assertEqual(depth.value, ["-1"])
+
+        revisions = browser.getControl(label="Include revisions")
+        self.assertEqual(revisions.selected, False)
+        revisions.selected = True
+
+        try:
+            # Plone 5.2
+            browser.getControl("Export").click()
+            contents = browser.contents
+        except LookupError:
+            # Plone 5.1 and lower
+            browser.getForm(index=1).submit()
+            if not browser.contents:
+                contents = DATA[-1]
+
+        # We should have gotten json.
+        data = json.loads(contents)
+        self.assertEqual(len(data), 3)
+
+        item = data[2]
+        self.assertEqual(item["@id"], portal.absolute_url() + "/folder1/doc2")
+
+        # check versions
+        self.assertIn("exportimport.versions", item)
+        versions = item["exportimport.versions"]
+        self.assertEqual(len(versions), 2)
+
+        # check first version
+        self.assertEqual(versions["0"]["title"], u"Document 2")
+        self.assertEqual(versions["0"]["description"], u"A Description")
+        self.assertEqual(versions["0"]["changeNote"], u"initial_version_changeNote")
+
+        # check version 2
+        self.assertEqual(versions["1"]["title"], u"Document 2 with changed title")
+        self.assertEqual(versions["1"]["description"], u"A Description")
+        self.assertEqual(versions["1"]["changeNote"], u"Föö bar")
+
+        # final/current version is the item itself
+        self.assertEqual(item["title"], u"Document 2 with changed title")
+        self.assertEqual(item["description"], u"New description in revision 3")
+        self.assertEqual(item["changeNote"], u"I am new!")
