@@ -6,6 +6,8 @@ from collective.exportimport.testing import (
 )
 from OFS.interfaces import IOrderedContainer
 from plone import api
+from plone.app.redirector.interfaces import IRedirectionStorage
+from plone.app.textfield.value import RichTextValue
 from plone.app.testing import login
 from plone.app.testing import SITE_OWNER_NAME
 from plone.app.testing import SITE_OWNER_PASSWORD
@@ -15,10 +17,14 @@ from plone.namedfile.file import NamedBlobImage
 from Products.CMFPlone.interfaces.constrains import ENABLED
 from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
 from Products.CMFPlone.tests import dummy
+from zope.annotation.interfaces import IAnnotations
+from zope.component import getUtility
+from zope.lifecycleevent import modified
 
 import json
 import os
 import shutil
+import six
 import tempfile
 import transaction
 import unittest
@@ -168,6 +174,7 @@ class TestImport(unittest.TestCase):
         self.assertIn("import local roles", lower_contents)
         self.assertIn("import default pages", lower_contents)
         self.assertIn("import object positions", lower_contents)
+        self.assertIn("import redirects", lower_contents)
 
     def test_import_content_document(self):
         # First create some content.
@@ -731,6 +738,60 @@ class TestImport(unittest.TestCase):
                 title=u"Collection 2",
             )
 
+    def test_import_workflow_history(self):
+        """workflow_history is imported last
+        """
+        # First create some content to export.
+        app = self.layer["app"]
+        portal = self.layer["portal"]
+        login(app, SITE_OWNER_NAME)
+        self.create_demo_content()
+        # modify a document
+        api.content.transition(to_state="published", obj=self.team)
+        history = self.team.workflow_history["simple_publication_workflow"]
+        self.assertEqual(len(history), 2)
+        self.assertEqual(history[1]["action"], "publish")
+        publish_time = history[1]["time"]
+        transaction.commit()
+
+        # Now export the complete portal.
+        browser = self.open_page("@@export_content")
+        browser.getControl(name="portal_type").value = [
+            "Folder",
+            "Image",
+            "Link",
+            "Document",
+        ]
+        browser.getForm(action="@@export_content").submit(name="submit")
+        contents = browser.contents
+        if not browser.contents:
+            contents = DATA[-1]
+
+        data = json.loads(contents)
+        self.assertEqual(len(data), 6)
+
+        # Remove the added content.
+        self.remove_demo_content()
+        transaction.commit()
+
+        # Now import it.
+        browser = self.open_page("@@import_content")
+        upload = browser.getControl(name="jsonfile")
+        upload.add_file(contents, "application/json", "Document.json")
+        browser.getForm(action="@@import_content").submit()
+        self.assertIn("Imported 6 items", browser.contents)
+
+        team = portal["about"]["team"]
+        history = team.workflow_history["simple_publication_workflow"]
+        self.assertEqual(len(history), 2)
+        self.assertEqual(history[1]["action"], "publish")
+        self.assertEqual(
+            history[1]["time"].asdatetime(),
+            publish_time.asdatetime().replace(microsecond=0))
+
+        api.content.transition(transition="retract", obj=team)
+        self.assertEqual(len(team.workflow_history["simple_publication_workflow"]), 3)
+
     def _disabled_test_import_blob_path(self):
         # This test is disabled, because the demo storage
         # has no 'fshelper' from which we can ask the blob path.
@@ -1029,3 +1090,180 @@ class TestImport(unittest.TestCase):
         new_doc = portal["doc1"]
         # the text should be the same
         self.assertEqual(new_doc.text.raw, old_text)
+
+    def test_import_redirects(self):
+        # First create some content.
+        app = self.layer["app"]
+        portal = self.layer["portal"]
+        login(app, SITE_OWNER_NAME)
+        api.content.create(
+            container=portal, type="Document", id="doc1", title="Document 1"
+        )
+        api.content.rename(obj=portal['doc1'], new_id='doc1-moved')
+        api.content.create(
+            container=portal, type="Document", id="doc2", title="Document 2"
+        )
+        api.content.rename(obj=portal['doc2'], new_id='doc2-moved')
+        transaction.commit()
+
+        # redirects are stored
+        storage = getUtility(IRedirectionStorage, context=portal)
+        self.assertTrue(storage.has_path('/plone/doc1'))
+        self.assertTrue(storage.has_path('/plone/doc2'))
+        self.assertEqual(storage.get('/plone/doc1'), '/plone/doc1-moved')
+        self.assertEqual(storage.get('/plone/doc2'), '/plone/doc2-moved')
+
+        # Export it.
+        browser = self.open_page("@@export_redirects")
+        browser.getForm(action="@@export_redirects").submit(name="form.submitted")
+        raw_data = browser.contents
+        if not browser.contents:
+            raw_data = DATA[-1]
+
+        self.assertTrue(raw_data)
+
+        # Now remove the redirects
+        storage = getUtility(IRedirectionStorage, context=portal)
+        storage.remove('/plone/doc1')
+        storage.remove('/plone/doc2')
+        transaction.commit()
+
+        # redirects are gone
+        storage = getUtility(IRedirectionStorage, context=portal)
+        self.assertFalse(storage.has_path('/plone/doc1'))
+        self.assertFalse(storage.has_path('/plone/doc2'))
+
+        # Now import it.
+        browser = self.open_page("@@import_redirects")
+        upload = browser.getControl(name="jsonfile")
+        upload.add_file(raw_data, "application/json", "redirects.json")
+        browser.getForm(action="@@import_redirects").submit(name="form.submitted")
+        self.assertIn("Redirects imported", browser.contents)
+
+        # redirects are back
+        storage = getUtility(IRedirectionStorage, context=portal)
+        self.assertTrue(storage.has_path('/plone/doc1'))
+        self.assertTrue(storage.has_path('/plone/doc2'))
+        self.assertEqual(storage.get('/plone/doc1'), '/plone/doc1-moved')
+        self.assertEqual(storage.get('/plone/doc2'), '/plone/doc2-moved')
+
+    def test_import_versions(self):
+        app = self.layer["app"]
+        portal = self.layer["portal"]
+        request = self.layer["request"]
+        login(app, SITE_OWNER_NAME)
+        if six.PY2:
+            # in Plone 4.3 this is somehow not set...
+            IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"initial_version_changeNote"
+        doc1 = api.content.create(
+            container=portal,
+            type="Document",
+            id="doc1",
+            title=u"Document 1",
+            description=u"A Description",
+        )
+        folder1 = api.content.create(
+            container=portal,
+            type="Folder",
+            id="folder1",
+            title=u"Folder 1",
+        )
+        doc2 = api.content.create(
+            container=folder1,
+            type="Document",
+            id="doc2",
+            title=u"Document 2",
+            description=u"A Description",
+        )
+        modified(doc1)
+        modified(folder1)
+        modified(doc2)
+
+        doc1.title= u"Document 1 with changed title"
+        modified(doc1)
+        doc2.title= u"Document 2 with changed title"
+        IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"Föö bar"
+        modified(doc2)
+
+        doc2.description= u"New description in revision 3"
+        IAnnotations(request)["plone.app.versioningbehavior-changeNote"] = u"I am new!"
+        modified(doc2)
+        folder1.title = u"Folder 1 with changed title"
+        modified(folder1)
+
+        transaction.commit()
+
+        repo_tool = api.portal.get_tool("portal_repository")
+        oldest = repo_tool.getHistory(doc2)._retrieve(doc2, 0, preserve=[], countPurged=False)
+        self.assertEqual(oldest.object.title, u"Document 2")
+
+        # Now export complete portal.
+        browser = self.open_page("@@export_content")
+        portal_type = browser.getControl(name="portal_type")
+        self.assertEqual(portal_type.value, [])
+        portal_type.value = ["Folder", "Document"]
+
+        depth = browser.getControl(name="depth")
+        self.assertEqual(depth.value, ["-1"])
+
+        revisions = browser.getControl(label="Include revisions")
+        self.assertEqual(revisions.selected, False)
+        revisions.selected = True
+
+        try:
+            # Plone 5.2
+            browser.getControl("Export").click()
+            contents = browser.contents
+        except LookupError:
+            # Plone 5.1 and lower
+            browser.getForm(index=1).submit()
+            if not browser.contents:
+                contents = DATA[-1]
+
+        # We should have gotten json.
+        data = json.loads(contents)
+        self.assertEqual(len(data), 3)
+
+        # Remove the added content.
+        api.content.delete(doc1)
+        api.content.delete(doc2)
+        api.content.delete(folder1)
+
+        transaction.commit()
+        self.assertNotIn("doc1", portal.contentIds())
+        self.assertNotIn("folder1", portal.contentIds())
+
+        # Now import it.
+        browser = self.open_page("@@import_content")
+        upload = browser.getControl(name="jsonfile")
+        upload.add_file(contents, "application/json", "plone.json")
+        form = browser.getForm(action="@@import_content")
+        self.assertFalse(form.getControl("Import all old revisions").selected)
+        form.getControl("Import all old revisions").selected = True
+        form.submit()
+        self.assertIn("Imported 3 items", browser.contents)
+
+        # The content should be back.
+        self.assertIn("doc1", portal.contentIds())
+        self.assertEqual(portal["folder1"].portal_type, "Folder")
+        doc2 = portal["folder1"]["doc2"]
+        self.assertEqual(doc2.title, u"Document 2 with changed title")
+        self.assertEqual(doc2.description, u"New description in revision 3")
+
+        history = repo_tool.getHistoryMetadata(doc2)
+        self.assertEqual(history.getLength(countPurged=True), 4)
+
+        if six.PY2:
+            # In 5.1 and 5.0 on py2 this test sometimes fails in gh-actions, locally they pass. Duh!
+            return
+
+        history_meta = history.retrieve(2)
+        self.assertEqual(history_meta["metadata"]["sys_metadata"]["comment"], u'Föö bar')
+
+        oldest = repo_tool.getHistory(doc2)._retrieve(doc2, 0, preserve=[], countPurged=False)
+        self.assertEqual(oldest.object.title, u"Document 2")
+
+        repo_tool.revert(portal["folder1"]["doc2"], 0)
+        doc2 = portal["folder1"]["doc2"]
+        self.assertEqual(doc2.title, u'Document 2')
+        self.assertEqual(doc2.description, u"A Description")
